# Author: Jeff Vogelsang <jeffvogelsang@gmail.com>
# Copyright 2013 Jeff Vogelsang
#
# Author: Mike Babineau <michael.babineau@gmail.com>
# Copyright 2011 Electronic Arts Inc.
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import requests
from device import LogglyDevice
from input import LogglyInput
from response import LogglyResponse
import json


class LogglyConnection(object):

    def __init__(self, username=None, password=None, domain=None, protocol="https"):

        # Note: LogglyConnection assumes HTTPS.
        self.protocol = protocol

        # Use environment variables for credentials if set.
        if None not in (os.environ.get('LOGGLY_DOMAIN'),
                        os.environ.get('LOGGLY_USERNAME'),
                        os.environ.get('LOGGLY_PASSWORD')):
            self.domain = os.environ.get('LOGGLY_DOMAIN')
            self.username = os.environ.get('LOGGLY_USERNAME')
            self.password = os.environ.get('LOGGLY_PASSWORD')
            # Protocol is optional. If we're using the system environment, pull it in.
            if os.environ.get('LOGGLY_PROTOCOL') is not None:
                self.protocol = os.environ.get('LOGGLY_PROTOCOL')

        # Use credentials passed to constructor over environment credentials.
        if None not in (username, password, domain):
            self.username = username
            self.password = password
            self.domain = domain

        # Raise error if we haven't managed to set the credentials at this point.
        if None in (getattr(self, 'username', None),
                    getattr(self, 'password', None),
                    getattr(self, 'domain', None)):
            raise AttributeError("No Loggly credentials provided or found in environment.")

        # Now set the base_url used for operations.
        self.base_url = '%s://%s/api' % (self.protocol, self.domain)

        # Authentication tuple for requests
        self.auth = (self.username, self.password)

    def __repr__(self):
        return "Connection:%s@%s" % (self.username, self.base_url)

    #### SOURCE MANAGEMENT API ####

    def _loggly_get(self, path):
        """Given a path, perform a get request using configured base_url and authentication."""

        response = requests.get("%s/%s" % (self.base_url, path), auth=self.auth)

        return LogglyResponse(response)

    def _loggly_post(self, path, data=None):
        """Given a path, perform a post request using configured base_url and authentication.

            If a dictionary containing post data provided send that with the post.
        """

        if data is None:
            response = requests.post("%s/%s" % (self.base_url, path), auth=self.auth)
        else:
            response = requests.post("%s/%s" % (self.base_url, path), data=data, auth=self.auth)

        return LogglyResponse(response)

    def _loggly_delete(self, path):
        """Given a path, perform a delete request using configured base_url and authentication."""

        response = requests.delete("%s/%s" % (self.base_url, path), auth=self.auth)

        return LogglyResponse(response)

    def get_all_inputs(self, input_names=None):
        """Get all inputs, or the specific inputs matching the supplied list of input names."""

        path = 'inputs/'

        response = self._loggly_get(path)

        json = response.json()
        loggly_inputs = []
        if input_names:
            for input_name in input_names:
                loggly_inputs += [LogglyInput(j) for j in json if j['name'] == input_name]
        else:
            loggly_inputs += [LogglyInput(j) for j in json]

        return loggly_inputs

    def get_input(self, input_id):
        """Get a specific input given its id."""

        path = 'inputs/%s/' % input_id

        response = self._loggly_get(path)

        json = response.json()
        loggly_input = LogglyInput(json)

        return loggly_input

    def list_inputs(self):
        """List all inputs."""

        loggly_inputs = self.get_all_inputs()

        input_list = [i.name for i in loggly_inputs]

        return input_list

    def create_input(self, name, service, input_format=None, description=None):
        """Create a new input given a name, service type, and optional description."""

        if not description: description = name

        # Note: Format is only relevant for HTTP sources.
        #       If format is omitted (none), Loggly defaults to text as the format.
        path = 'inputs/'
        data = {'name': name, 'service': service, 'format': input_format, 'description': description}

        response = self._loggly_post(path, data)

        json = response.json()
        loggly_input = LogglyInput(json)

        return loggly_input

    def delete_input(self, loggly_input):
        """Delete the given input."""

        path = 'inputs/%s/' % loggly_input.id

        response = self._loggly_delete(path)

        return "%s:%s" % (response.status_code, response.text)

    def get_all_devices(self, device_names=None):
        """Get all devices, or the specific devices matching the supplied list of device names."""

        path = 'devices/'

        response = self._loggly_get(path)

        json = response.json()
        loggly_devices = []
        if device_names:
            for device_name in device_names:
                loggly_devices += [LogglyDevice(j) for j in json if j['ip'] == device_name]
        else:
            loggly_devices += [LogglyDevice(j) for j in json]

        return loggly_devices

    def get_device(self, device_id):
        """Get a specific device given its id."""

        path = 'devices/%s/' % device_id

        response = self._loggly_get(path)

        json = response.json()
        loggly_device = LogglyDevice(json)

        return loggly_device

    def list_devices(self):
        """List all devices."""

        loggly_devices = self.get_all_devices()

        device_list = [i.name for i in loggly_devices]

        return device_list

    def add_device_to_input(self, loggly_device, loggly_input):
        """Add an arbitrary device (specified IP address) to the given input."""

        path = 'devices/'

        data = {'input_id': loggly_input.id, 'ip': loggly_device.ip}

        response = self._loggly_post(path, data)

        json = response.json()
        loggly_device = LogglyDevice(json)

        return loggly_device

    def add_this_device_to_input(self, loggly_input):
        """Add a device matching the IP of the HTTP client calling the API from the given input."""

        path = 'inputs/%s/adddevice/' % loggly_input.id

        response = self._loggly_post(path)

        json = response.json()
        loggly_device = LogglyDevice(json)

        return loggly_device

    def remove_this_device_from_input(self, loggly_input):
        """Remove the device matching the IP of the HTTP client calling the API from the given input."""

        path = 'inputs/%s/removedevice/' % loggly_input.id

        response = self._loggly_delete(path)

        return "%s:%s" % (response.status_code, response.text)

    def delete_device(self, loggly_device):
        """Remove the specified device.

        Note: This removes a device from all inputs. Compare with remove_this_device_from_input.
        """

        path = 'devices/%s/' % loggly_device.id

        response = self._loggly_delete(path)

        return "%s:%s" % (response.status_code, response.text)

    #### SUBMISSION API ####

    def _submit_data(self, input_key, data, data_type="text"):
        """Submit data to input defined by input_key.

        Note: While this is similar _loggly_post(), it goes to a different URL and has different
              requirements for headers. Thus it has its own method.
        """

        # The content-type header differentiates between text and json. Text is the default.
        if data_type == "json":
            headers = {'content-type': 'application/x-www-form-urlencoded'}
        else:
            headers = {'content-type': 'text/plain'}

        # Note that the URL for HTTP inputs is the same for all customers...
        url = "https://logs.loggly.com/inputs/%s" % input_key

        response = requests.post(url, data=data, headers=headers, auth=self.auth)

        return LogglyResponse(response)

    def submit_text_data(self, text_data, input_key):
        """Submit plain text data to HTTP input identified by input_key."""

        response = self._submit_data(input_key, text_data)

        return "%s:%s" % (response.status_code, response.text)

    def submit_json_data(self, json_data, input_key):
        """Submit JSON formatted data to HTTP input identified by input_key."""

        response = self._submit_data(input_key, json_data, "json")

        return "%s:%s" % (response.status_code, response.text)

    #### RETRIEVAL API ####

    # Standard Queries

    def _search_events(self, query_string, rows=None, start=None, from_date=None, until_date=None,
                       order=None, callback=None, output_format=None, fields=None):
        """Assemble Loggly API request for searching events data and returning results."""

        query_params = {
            'q': query_string
        }

        if rows is not None:
            query_params['rows'] = rows
        if start is not None:
            query_params['start'] = start
        if from_date is not None:
            query_params['from'] = from_date
        if until_date is not None:
            query_params['until'] = until_date
        if order is not None:
            query_params['order'] = order
        if callback is not None:
            query_params['callback'] = callback
        if output_format is not None:
            query_params['format'] = output_format
        if fields is not None:
            query_params['fields'] = fields

        path = "search?"

        for param in query_params:
            path = path + param + "=" + query_params[param] + "&"
        path = path.rstrip("&")  # remove extra &

        response = requests.get("%s/%s" % (self.base_url, path), auth=self.auth)

        return LogglyResponse(response)

    def get_events(self, query_string, **kwargs):
        """Return events matching query_string."""

        response = self._search_events(query_string, **kwargs)

        return response.text

    def get_events_json(self, query_string, **kwargs):
        """Return events matching query_string as (raw) JSON."""

        response = self._search_events(query_string, output_format="json", **kwargs)

        return response.text

    def get_events_dict(self, query_string, **kwargs):
        """Return events matching query_string as a Python dictionary."""

        return json.loads(self.get_events_json(query_string, **kwargs))

    def get_events_xml(self, query_string, **kwargs):
        """Return events matching query_string as XML."""

        response = self._search_events(query_string, output_format="xml", **kwargs)

        return response.text

    def get_events_text(self, query_string, **kwargs):
        """Return events matching query_string as Text."""

        response = self._search_events(query_string, output_format="text", **kwargs)

        return response.text

    # Faceted Queries

    def _search_events_faceted(self, facetby, query_string, from_date=None, until_date=None, buckets=None,
                               gap=None, callback=None, output_format=None):

        query_params = {
            'q': query_string
        }

        if from_date is not None:
            query_params['from'] = from_date
        if until_date is not None:
            query_params['until'] = until_date
        if buckets is not None:
            query_params['buckets'] = buckets
        if gap is not None:
            query_params['gap'] = gap
        if callback is not None:
            query_params['callback'] = callback
        if output_format is not None:
            query_params['format'] = output_format

        path = "facets/%s/?" % facetby

        for param in query_params:
            path = path + param + "=" + query_params[param] + "&"
        path = path.rstrip("&")  # remove extra &

        response = requests.get("%s/%s" % (self.base_url, path), auth=self.auth)

        return LogglyResponse(response)

    def get_events_faceted(self, facetby, query_string, **kwargs):
        """Return faceted events matching query_string."""

        response = self._search_events_faceted(facetby, query_string, **kwargs)

        return response.text

    def get_events_faceted_json(self, facetby, query_string, **kwargs):
        """Return faceted events matching query_string as JSON."""

        response = self._search_events_faceted(facetby, query_string, output_format="json", **kwargs)

        return response.text

    def get_events_faceted_dict(self, facetby, query_string, **kwargs):
        """Return faceted events matching query_string as a Python dictionary."""

        return json.loads(self.get_events_faceted_json(facetby, query_string, **kwargs))

    def get_events_faceted_xml(self, facetby, query_string, output_format="xml", **kwargs):
        """Return faceted events matching query_string as XML."""

        response = self._search_events_faceted(facetby, query_string, **kwargs)

        return response.text

    def get_events_faceted_text(self, facetby, query_string, **kwargs):
        """Return faceted events matching query_string as Text."""

        response = self._search_events_faceted(facetby, query_string, output_format="text", **kwargs)

        return response.text


# Author: Jeff Vogelsang <jeffvogelsang@gmail.com>
# Copyright 2013 Jeff Vogelsang
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from error import LogglyException


class LogglyResponse(object):
    """Wrapper for the requests package response object.

    Provides a mechanism for injecting Loggly-specific response information,
    and handling response status_codes from Loggly that are considered errors.
    """

    # Loggly-specific response codes to API-calls. See: http://loggly.com/support/advanced/api-manage-sources/
    status_codes = {200: {'message': 'OK',
                          'description': 'Indicates that the request was successful.'},
                    201: {'message': 'Created',
                          'description': 'The object was successfully created. This is for a POST call.'},
                    204: {'message': 'Deleted',
                          'description': 'The object was deleted. This pertains to DELETE calls.'},
                    400: {'message': 'Bad Request',
                          'description': 'Check your request parameters. You might be using an unsupported '
                                         'parameter or have a malformed something or another.'},
                    401: {'message': 'Unauthorized',
                          'description': 'Either your credentials specified were invalid.'},
                    403: {'message': 'Forbidden',
                          'description': 'User does not have privileges to execute the action.'},
                    404: {'message': 'Not Found',
                          'description': 'The resource you have referenced could not be found.'},
                    409: {'message': 'Conflict/Duplicate',
                          'description': 'There was some conflict. Most likely you are '
                                         'trying to create a resource that already exists.'},
                    410: {'message': 'Gone',
                          'description': 'You have referenced an object that does not exist.'},
                    500: {'message': 'Internal Server Error',
                          'description': 'There has been an error from which Loggly could not '
                                         'recover. We are likely notified when this happens.'},
                    501: {'message': 'Not Implemented',
                          'description': 'You are trying to access functionality that '
                                         'is not implemented. Yet.'},
                    503: {'message': 'Throttled',
                          'description': 'Like a needy child, you are overloading '
                                         'us with requests for events. Try again later.'}}

    # Codes considered exceptions/failures
    exception_codes = (400, 401, 403, 404, 409, 410, 500, 501, 503)

    def __init__(self, obj):
        self._wrapped_obj = obj

        # Add Loggly-specific information to response.
        self.loggly_message = self.status_codes[self.status_code]['message']
        self.loggly_description = self.status_codes[self.status_code]['description']
        self.loggly_info = "%s: %s - %s" % (self.status_code, self.loggly_message, self.loggly_description)

        # Handle responses considered to be errors.
        if self.status_code in self.exception_codes:
            raise LogglyException(self.loggly_info)

    def __getattr__(self, attr):
        if attr in self.__dict__:
            return getattr(self, attr)
        return getattr(self._wrapped_obj, attr)

    def __repr__(self):
        # Add Loggly-specific information to representation.
        return "Response: %s" % self.loggly_info


# Author: Jeff Vogelsang <jeffvogelsang@gmail.com>
# Copyright 2013 Jeff Vogelsang
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import random
from random import randrange
import json
import string
import unittest
import time
from eureka import connect_loggly
from connection import LogglyDevice

# Ensure that live test only run if LOGGLY_TEST_LIVE variable is present in the environment and set to 'True'
# Note: Live tests are designed to reasonably safely create and destroy Loggly inventory without affecting
#       existing configuration through use of randomized strings and loop-back IP addresses.
enable_live_tests = os.environ.get('LOGGLY_TEST_LIVE')
if enable_live_tests is not None and enable_live_tests == 'True':
    enable_live_tests = True


def rand_string(count=12):
    """Return random string of length count with letters and numbers, mixed case. Uses Python randomness."""

    return ''.join(random.choice(string.ascii_letters + string.digits) for x in range(count))


def get_rand_private_ip():
    """Return a random IP based on the 127.x.x.x block."""

    return "127.%s.%s.%s" % (randrange(0, 255, 1), randrange(0, 255, 1), randrange(0, 255, 1))


class TestLoggly(unittest.TestCase):

    def setUp(self):

        # Preserve environment settings, put them back when done.
        self.env_username_save = os.environ.get('LOGGLY_USERNAME')
        self.env_password_save = os.environ.get('LOGGLY_PASSWORD')
        self.env_domain_save = os.environ.get('LOGGLY_DOMAIN')
        self.env_protocol_save = os.environ.get('LOGGLY_PROTOCOL')

    def tearDown(self):

        def restore_environment(env_var, env_var_saved):

            if env_var_saved is not None:
                os.environ[env_var] = env_var_saved
            else:
                if os.environ.get(env_var) is not None:
                    del os.environ[env_var]

        restore_environment('LOGGLY_USERNAME', self.env_username_save)
        restore_environment('LOGGLY_PASSWORD', self.env_password_save)
        restore_environment('LOGGLY_DOMAIN', self.env_domain_save)
        restore_environment('LOGGLY_PROTOCOL', self.env_protocol_save)

    def testConnCredsFromEnv(self):

        os.environ['LOGGLY_USERNAME'] = 'env_username'
        os.environ['LOGGLY_PASSWORD'] = 'env_password'
        os.environ['LOGGLY_DOMAIN'] = 'env_domain'

        conn = connect_loggly()

        self.assertEquals('env_username', getattr(conn, 'username'))
        self.assertEquals('env_password', getattr(conn, 'password'))
        self.assertEquals('https://env_domain/api', getattr(conn, 'base_url'))

        # Make sure we can override the HTTP default.
        os.environ['LOGGLY_PROTOCOL'] = 'http'

        conn = connect_loggly()
        self.assertEquals('http', getattr(conn, 'protocol'))
        self.assertEquals('http://env_domain/api', getattr(conn, 'base_url'))

    def testConnCredsSupplied(self):

        conn = connect_loggly('username', 'password', 'domain')

        self.assertEquals('username', getattr(conn, 'username'))
        self.assertEquals('password', getattr(conn, 'password'))
        self.assertEquals('https://domain/api', getattr(conn, 'base_url'))

        conn = connect_loggly('username', 'password', 'domain', 'http')
        self.assertEquals('http', getattr(conn, 'protocol'))
        self.assertEquals('http://domain/api', getattr(conn, 'base_url'))

    def testConnCredsMissing(self):

        del os.environ['LOGGLY_USERNAME']
        del os.environ['LOGGLY_PASSWORD']
        del os.environ['LOGGLY_DOMAIN']

        self.assertRaises(AttributeError, connect_loggly)

    def testConnRepr(self):

        os.environ['LOGGLY_USERNAME'] = 'env_username'
        os.environ['LOGGLY_PASSWORD'] = 'env_password'
        os.environ['LOGGLY_DOMAIN'] = 'env_domain'

        # Credentials from enviornment
        conn = connect_loggly()
        self.assertEqual("Connection:env_username@https://env_domain/api", "%s" % conn)

        del os.environ['LOGGLY_USERNAME']
        del os.environ['LOGGLY_PASSWORD']
        del os.environ['LOGGLY_DOMAIN']

        # Credentials supplied to constructor
        conn = connect_loggly('username', 'password', 'domain')
        self.assertEqual("Connection:username@https://domain/api", "%s" % conn)


@unittest.skipIf(not enable_live_tests, 'Live connection tests skipped.')
class TestLogglyLive(unittest.TestCase):
    """Live tests. Prove code works against live API.

    Note: As these are live tests, running an integration-level, they are subject to environmental failures!
          If you experience a failure, the tests may leave behind inputs and devices you'll want to clean up.
    """

    def setUp(self):
        """Re-use a live connection to loggly for tests."""

        self.conn = connect_loggly()

        print "Using: %s" % self.conn

    # Helper methods

    def create_input(self, input_type="syslogudp", input_format="text"):
        """Create and with a randomized name and description for testing purposes."""

        input_name = "test-input-%s" % rand_string()
        input_desc = "test-description-%s" % rand_string()

        loggly_input = self.conn.create_input(input_name, input_type, input_format, input_desc)

        print "Created input: %s, %s" % (loggly_input.id, loggly_input.name)
        return loggly_input

    def create_syslog_input(self):
        """Create a syslog input with a randomized named and description for testing purposes."""

        return self.create_input(input_type="syslogudp")

    def create_http_text_input(self):
        """Create a http text input with a randomized named and description for testing purposes."""

        return self.create_input(input_type="http")

    def create_http_json_input(self):
        """Create a http json input with a randomized named and description for testing purposes."""

        return self.create_input(input_type="http", input_format="json")

    # Tests

    def testCreateDeleteInput(self):
        """Create an input then delete it."""

        loggly_input = self.create_syslog_input()
        self.conn.delete_input(loggly_input)

    def testCreateDeleteDevice(self):
        """Create a device then delete it.

        This requires adding the device to an input, so we create and delete one of these as well.
        """

        loggly_input = self.create_syslog_input()

        min_loggly_device = LogglyDevice({'ip': get_rand_private_ip()}) # de minimus Loggly device
        loggly_device = self.conn.add_device_to_input(min_loggly_device, loggly_input)  # create actual device

        self.conn.delete_device(loggly_device)
        self.conn.delete_input(loggly_input)

    def testCreateDeleteThisDevice(self):
        """Create a device based on the current IP that Loggly sees, then delete it.

        This requires adding the device to an input, so we create and delete one of these as well.
        """

        loggly_input = self.create_syslog_input()

        loggly_device = self.conn.add_this_device_to_input(loggly_input)

        self.conn.remove_this_device_from_input(loggly_input)
        self.conn.delete_device(loggly_device)

    def testGetAllInputs(self):
        """Get all inputs.

        To make sure we're getting multiple inputs, create a few, get the list, then delete them.
        """
        loggly_input1 = self.create_syslog_input()
        loggly_input2 = self.create_syslog_input()

        inputs = self.conn.get_all_inputs()
        self.assertGreaterEqual(len(inputs), 2)

        self.conn.delete_input(loggly_input1)
        self.conn.delete_input(loggly_input2)

    def testGetInputFromGetAllInputs(self):
        """Use get all inputs to get a specific input by name.

        We create a input so we can test finding a specific input, then delete it.
        """

        loggly_input1 = self.create_syslog_input()
        loggly_input2 = self.create_syslog_input()

        self.assertEqual(1, len(self.conn.get_all_inputs([loggly_input1.name])))
        self.assertEqual(loggly_input1.id, self.conn.get_all_inputs([loggly_input1.name])[0].id)

        self.assertEqual(2, len(self.conn.get_all_inputs([loggly_input1.name, loggly_input2.name])))

        self.conn.delete_input(loggly_input1)
        self.conn.delete_input(loggly_input2)

    def testGetInput(self):
        """Get a single input by id.

        We create a input so we can test finding a specific input, then delete it.
        """

        loggly_input_to_find = self.create_syslog_input()
        loggly_input_found = self.conn.get_input(loggly_input_to_find.id)

        self.assertEqual(loggly_input_found.id, loggly_input_to_find.id)

        self.conn.delete_input(loggly_input_found)

    def testGetAllDevices(self):
        """Get all devices.

        To make sure we're getting multiple devices, create a few attached to a new input, get the list,
          then delete the input and the devices.
        """

        loggly_input = self.create_syslog_input()

        min_loggly_device1 = LogglyDevice({'ip': get_rand_private_ip()}) # de minimus Loggly device
        min_loggly_device2 = LogglyDevice({'ip': get_rand_private_ip()})

        loggly_device1 = self.conn.add_device_to_input(min_loggly_device1, loggly_input) # create actual devices
        loggly_device2 = self.conn.add_device_to_input(min_loggly_device2, loggly_input)

        devices = self.conn.get_all_devices()

        self.assertGreaterEqual(len(devices), 2)

        self.conn.delete_device(loggly_device1)
        self.conn.delete_device(loggly_device2)
        self.conn.delete_input(loggly_input)

    def testGetDeviceFromGetAllDevices(self):
        """Use get all devices to get a specific device by IP.

        We create an input and a device so we can test finding a specific device, then delete them.
        """

        loggly_input = self.create_syslog_input()

        min_loggly_device1 = LogglyDevice({'ip': get_rand_private_ip()}) # de minimus Loggly device
        min_loggly_device2 = LogglyDevice({'ip': get_rand_private_ip()})

        loggly_device1 = self.conn.add_device_to_input(min_loggly_device1, loggly_input) # create actual devices
        loggly_device2 = self.conn.add_device_to_input(min_loggly_device2, loggly_input)

        self.assertEqual(1, len(self.conn.get_all_devices([loggly_device1.ip])))
        self.assertEqual(loggly_device1.id, self.conn.get_all_devices([loggly_device1.ip])[0].id)

        self.assertEqual(2, len(self.conn.get_all_devices([loggly_device1.ip, loggly_device2.ip])))

        self.conn.delete_device(loggly_device1)
        self.conn.delete_device(loggly_device2)
        self.conn.delete_input(loggly_input)

    def testGetDevice(self):
        """ Get a single device by id.

        We create a device so we can test finding a specific device, then delete it.
        """

        loggly_input = self.create_syslog_input()

        min_loggly_device = LogglyDevice({'ip': get_rand_private_ip()}) # de minimus Loggly device
        loggly_device_to_find = self.conn.add_device_to_input(min_loggly_device, loggly_input) # create actual devices

        loggly_device_found = self.conn.get_device(loggly_device_to_find.id)
        self.assertEqual(loggly_device_found.id, loggly_device_to_find.id)

        self.conn.delete_device(loggly_device_found)
        self.conn.delete_input(loggly_input)

    # Next four tests have repetitive code. An opportunity for later cleanup.

    def testSubmitAndRetrieveTextEvents(self):
        """ Submit some text data, and then find it.

        We create an HTTP text input, submit some unique data, and then find it.
        """

        # Create an input. Need an HTTP input.
        loggly_input = self.create_http_text_input()

        # Make a random string that we're certain won't be found.
        string_event = rand_string(150)

        # Test submitting a Text event.
        submit_attempts = 10
        submit_attempt_delay = 30
        event_submitted = False
        while not event_submitted and submit_attempts > 0:
            try:
                self.conn.submit_text_data(string_event, loggly_input.input_token)
                print "Event submitted."
                event_submitted = True
            except Exception as e:
                submit_attempts -= 1
                print "Error submitting event: %s" % e.message
                print "%s tries left. Will try again in %s seconds." % (submit_attempts, submit_attempt_delay)
                time.sleep(submit_attempt_delay)

        self.assertTrue(event_submitted, "Event not submitted.")

        # Test retrieving a Text event.
        search_attempts = 10
        search_attempt_delay = 30
        event_found = False
        while not event_found and search_attempts > 0:
            try:
                events = self.conn.get_events_dict(string_event)
                num_found = events['numFound']
                if num_found > 0:
                    print "Event found."
                    event_found = True
                else:
                    search_attempts -= 1
                    print "Event not found. %s tries left. Will try again in %s seconds."\
                          % (search_attempts, search_attempt_delay)
                    time.sleep(search_attempt_delay)
            except Exception as e:
                search_attempts -= 1
                print "Error searching for event: %s" % e.message
                print "%s tries left. Will try again in %s seconds." % (search_attempts, search_attempt_delay)

        self.assertTrue(event_found, "Event not found.")

        # Remove the input
        self.conn.delete_input(loggly_input)

    def testSubmitAndRetrieveJsonEvents(self):

        # Create an input. Need an HTTP input.
        loggly_input = self.create_http_json_input()

        # Make a random string that we're certain won't be found.
        event_string = rand_string(150)
        event = {
            'event_string': event_string
        }
        json_event = json.dumps(event)

        # Test submitting a JSON event.
        submit_attempts = 10
        submit_attempt_delay = 30
        event_submitted = False
        while not event_submitted and submit_attempts > 0:
            try:
                self.conn.submit_text_data(json_event, loggly_input.input_token)
                print "Event submitted."
                event_submitted = True
            except Exception as e:
                submit_attempts -= 1
                print "Error submitting event: %s" % e.message
                print "%s tries left. Will try again in %s seconds." % (submit_attempts, submit_attempt_delay)
                time.sleep(submit_attempt_delay)

        self.assertTrue(event_submitted, "Event not submitted.")

        # Test retrieving a JSON event.
        search_attempts = 10
        search_attempt_delay = 30
        event_found = False
        while not event_found and search_attempts > 0:
            try:
                events = self.conn.get_events_dict('json.event_string:"%s"' % event_string)
                num_found = events['numFound']
                if num_found > 0:
                    print "Event found."
                    event_found = True
                else:
                    search_attempts -= 1
                    print "Event not found. %s tries left. Will try again in %s seconds." \
                          % (search_attempts, search_attempt_delay)
                    time.sleep(search_attempt_delay)
            except Exception as e:
                search_attempts -= 1
                print "Error searching for event: %s" % e.message
                print "%s tries left. Will try again in %s seconds." % (search_attempts, search_attempt_delay)

        self.assertTrue(event_found, "Event not found.")

        # Remove the input
        self.conn.delete_input(loggly_input)

    def testSubmitAndRetrieveTextEventsFaceted(self):

        # Create an input. Need an HTTP input.
        loggly_input = self.create_http_json_input()

        # Make a random string that we're certain won't be found.
        string_event = rand_string(150)

        # Test submitting a Text event.
        submit_attempts = 10
        submit_attempt_delay = 30
        event_submitted = False
        while not event_submitted and submit_attempts > 0:
            try:
                self.conn.submit_text_data(string_event, loggly_input.input_token)
                print "Event submitted."
                event_submitted = True
            except Exception as e:
                submit_attempts -= 1
                print "Error submitting event: %s" % e.message
                print "%s tries left. Will try again in %s seconds." % (submit_attempts, submit_attempt_delay)
                time.sleep(submit_attempt_delay)

        self.assertTrue(event_submitted, "Event not submitted.")

        # Test retrieving a Text event.
        search_attempts = 10
        search_attempt_delay = 30
        event_found = False
        while not event_found and search_attempts > 0:
            try:
                events = self.conn.get_events_faceted_dict("date", string_event)
                num_found = events['numFound']
                if num_found > 0:
                    print "Event found."
                    event_found = True
                else:
                    search_attempts -= 1
                    print "Event not found. %s tries left. Will try again in %s seconds." \
                          % (search_attempts, search_attempt_delay)
                    time.sleep(search_attempt_delay)
            except Exception as e:
                search_attempts -= 1
                print "Error searching for event: %s" % e.message
                print "%s tries left. Will try again in %s seconds." % (search_attempts, search_attempt_delay)

        self.assertTrue(event_found, "Event not found.")

        # Remove the input
        self.conn.delete_input(loggly_input)

    def testSubmitAndRetrieveJsonEventsFaceted(self):

        # Create an input. Need an HTTP input.
        loggly_input = self.create_http_json_input()

        # Make a random string that we're certain won't be found.
        event_string = rand_string(150)
        event = {
            'event_string': event_string
        }
        json_event = json.dumps(event)

        # Test submitting a JSON event.
        submit_attempts = 10
        submit_attempt_delay = 30
        event_submitted = False
        while not event_submitted and submit_attempts > 0:
            try:
                self.conn.submit_text_data(json_event, loggly_input.input_token)
                print "Event submitted."
                event_submitted = True
            except Exception as e:
                submit_attempts -= 1
                print "Error submitting event: %s" % e.message
                print "%s tries left. Will try again in %s seconds." % (submit_attempts, submit_attempt_delay)
                time.sleep(submit_attempt_delay)

        self.assertTrue(event_submitted, "Event not submitted.")

        # Test retrieving a JSON event.
        search_attempts = 10
        search_attempt_delay = 30
        event_found = False
        while not event_found and search_attempts > 0:
            try:
                events = self.conn.get_events_faceted_dict("date", 'json.event_string:"%s"' % event_string)
                num_found = events['numFound']
                if num_found > 0:
                    print "Event found."
                    event_found = True
                else:
                    search_attempts -= 1
                    print "Event not found. %s tries left. Will try again in %s seconds." \
                          % (search_attempts, search_attempt_delay)
                    time.sleep(search_attempt_delay)
            except Exception as e:
                search_attempts -= 1
                print "Error searching for event: %s" % e.message
                print "%s tries left. Will try again in %s seconds." % (search_attempts, search_attempt_delay)

        self.assertTrue(event_found, "Event not found.")

        # Remove the input
        self.conn.delete_input(loggly_input)

    def testLogglyExceptions(self):

        # A device with an 12-character string id should cause a 400 status code and raise and exception.
        bad_device = LogglyDevice({'id': rand_string()})

        self.assertRaises(Exception, self.conn.delete_device, bad_device)


if __name__ == '__main__':

    unittest.main(verbosity=2)


# Author: Jeff Vogelsang <jeffvogelsang@gmail.com>
# Copyright 2013 Jeff Vogelsang
#
# Author: Mike Babineau <michael.babineau@gmail.com>
# Copyright 2011 Electronic Arts Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from connection import LogglyConnection

__version__ = '0.10'

__author__ = 'Jeff Vogelsang <jeffvogelsang@gmail.com>'
__copyright__ = "Jeff Vogelsang"
__license__ = "Apache v2.0"


def connect_loggly(username=None, password=None, domain=None, protocol="https"):
    """Get a Loggly connection."""

    # Note: connect_loggly assumes HTTPS.

    return LogglyConnection(username, password, domain, protocol)
